% SUM-MAX MAPS FOR TEMPLATE MATCHING INFERENCE
%% load in learned template, set parameter values
clear; close all; 
load 'learnedTemplate'; % load in the learned template
mex Cfind.c;   % compile C code for template matching inference
%% load in test image, generate multiple resolutions
numResolution = 10;  % total number of resolutions to search 
allSizex = zeros(1, numResolution); allSizey = zeros(1, numResolution); 
tmpIm = imread('test.jpg'); 
if size(tmpIm,3) == 3
     tmpIm = rgb2gray(tmpIm);
end
J0 = single(tmpIm); 
MAX1maps = cell(numResolution, numOrient); 
SUM2map = cell(1, numResolution); 
for(img=1:numResolution)
    J{img} = imresize(J0, .24+(img-1)*.03, 'nearest');  % scaled images
    [sizex, sizey] = size(J{img}); 
    allSizex(img) = sizex; allSizey(img) = sizey; % sizes of images at multiple resolutions
    sizexSubsample = floor(sizex/subsample); sizeySubsample = floor(sizey/subsample); 
    SUM2map{img} = single(zeros(sizexSubsample, sizeySubsample));  % log-likelihood score maps for testing images
    for (orient=1:numOrient)
        MAX1map{img, orient} = single(zeros(sizexSubsample, sizeySubsample)); 
    end
    translatedTemplate{img} = single(zeros(sizex, sizey));  % symbolic sketch on testing image
end
%% compute SUM1 maps for all resolutions
disp(['start filtering']); tic
SUM1map = ApplyFilterfft(J, allFilter, localOrNot, localHalfx, localHalfy, thresholdFactor);   % filter testing image at all resolutions
disp(['filtering time: ' num2str(toc) ' seconds']);
MAX2score = single(zeros(1, numResolution));  % maximum log-likelihood score at each resolution
%% detect the object and superpose template
disp(['start finding']); tic
Cfind(numResolution, allSizex, allSizey, numOrient, ...
      locationShiftLimit, orientShiftLimit, subsample, saturation,  ...
      halfFilterSize, SUM1map, allSymbol(1, :), ...
      numElement, selectedOrient, selectedx, selectedy, selectedlambda, selectedLogZ, ...
      MAX1map, SUM2map, translatedTemplate, MAX2score);
disp(['mex-C finding time: ' num2str(toc) ' seconds']);
%% display results
[maxOverResolution ind] = max(MAX2score);   % most likely resolution
towrite = tmpIm; 
outepsgif(towrite, 'ex2_3_original_LNA'); 
for(i=1:numResolution) 
  towrite = J{i}+translatedTemplate{i}*100;  
  outepsgif(towrite, ['ex2_3_scale_LNA' num2str(i)]); 
end

figure('Position', [1 1 1024 768]); 
surf(double(SUM2map{ind})); % log-likelihood score map at optimal resolution
% adjust the display format and save the plot
set(gca,'FontSize',18);
set(gcf, 'PaperPositionMode', 'auto');
print('-depsc', 'ex2_3_S2_LNA.eps');

figure; h1=plot(MAX2score); 
title('MAX2 vs resolution'); xlabel('resolution'); ylabel('MAX2');
% adjust the display format and save the plot
set(gca,'FontSize',16);
set(get(gca,'XLabel'),'FontSize',18);
set(get(gca,'YLabel'),'FontSize',18);
set(get(gca,'Title'),'FontSize',20);
set(h1,'Linewidth',3);
set(gcf, 'PaperPositionMode', 'auto');
print('-depsc', 'ex2_3_M2_LNA.eps');







